@@ -343,25 +343,162 @@ class OracleTools:
         '''
         return dba_conn.run(sql)
     
+    @staticmethod   
+    def expdp_estimate(host_conn, sid, users='', estimate='statistics'):
+        """ 评估导出的dmp文件大小, users不填会使用full=y, estimate=statistics|blocks """
+        date_str = tools.locatdate()
+        ip = host_conn.host
+        if users == '':
+            sh = '''
+                source ~/.bash_profile
+                export ORACLE_SID=''' + sid + '''
+                expdp \\'/ as sysdba\\' \\
+                compression=all \\
+                cluster=n \\
+                parallel=8 \\
+                full=y \\
+                estimate_only=y \\
+                estimate=''' + estimate + '''
+            '''
+        else:
+            sh = '''
+                source ~/.bash_profile
+                export ORACLE_SID=''' + sid + '''
+                expdp \\'/ as sysdba\\' \\
+                compression=all \\
+                cluster=n \\
+                parallel=8 \\
+                schemas=''' + users + ''' \\
+                estimate_only=y \\
+                estimate=''' + estimate + '''
+            '''  
+        mess = host_conn.exec_script(sh)
+        size = None
+        rs_list = []
+        if 'successfully completed' in mess:
+            size = mess.split('method: ')[-1].split('\n')[0]
+            lines = mess.replace('\n', '').split('.  estimated')
+            for line in lines:
+                if 'Total' in line or 'expdp' in line:
+                    pass
+                else:
+                    line = tools.merge_spaces(line.replace('"', '')).strip()
+                    user_name = line.split(' ')[0].split('.')[0]
+                    obj_name = line.split(' ')[0].split('.')[1]
+                    obj_size = line.split(' ')[1]
+                    dw = line.split(' ')[2]
+                    if ':' in obj_name:
+                        fq_name = obj_name.split(':')[1]
+                        tab_name = obj_name.split(':')[0]
+                    else:
+                        tab_name = obj_name
+                        fq_name = tab_name
+                    if dw == 'GB':
+                        obj_size = float(obj_size) * 1024
+                    elif dw == 'KB':
+                        obj_size = float(obj_size) / 1024
+                    rs_list.append((date_str, ip, sid, user_name, tab_name, fq_name, obj_size))
+        elif 'elapsed 0' in mess:
+            size = mess.split('TATISTICS : ')[-1].split('\n')[0]
+            lines = mess.split('\n')
+            
+            for line in lines:
+                if '.   "' in line:
+                    line = tools.merge_spaces(line.replace('"', '').replace('\n', '')).strip()
+                    user_name = line.split(' ')[1].split('.')[0]
+                    obj_name = line.split(' ')[1].split('.')[1]
+                    obj_size = line.split(' ')[2]
+                    dw = line.split(' ')[3]
+                    if ':' in obj_name:
+                        fq_name = obj_name.split(':')[1]
+                        tab_name = obj_name.split(':')[0]
+                    else:
+                        tab_name = obj_name
+                        fq_name = tab_name
+                    if dw == 'GB':
+                        obj_size = float(obj_size) * 1024
+                    elif dw == 'KB':
+                        obj_size = float(obj_size) / 1024
+                    rs_list.append((date_str, ip, sid, user_name, tab_name, fq_name, obj_size))
+                    
+        size = size.replace('\n', '').strip()
+        return size, rs_list, mess
+    
+    @staticmethod  
+    def expdp(src_host_conn, src_dba_conn, src_ora_sid, src_ora_user='', dump_tmp_py='/tmp/oracle_dump_py'):
+        if src_ora_user == '':
+            src_ora_user = 'all'
+        date_str = tools.locatdate()
+        dump_filename = src_ora_user + '_' + date_str + '.dmp'
+        log = tools.Log('logs/expdp.log')
+        src_ora_user = ''
+        
+        # 源端创建DUMP_TMP_PY
+        src_host_ip = src_host_conn.host
+        try:
+            src_dba_conn.run('drop directory DUMP_TMP_PY')
+        except Exception as e:
+            pass
+        try:
+            src_dba_conn.run("create directory DUMP_TMP_PY as '" + dump_tmp_py + "'")
+            log.info('源端', src_host_ip, 'DUMP_TMP_PY 创建成功')
+        except Exception as e:
+            log.error('源端', src_host_ip, 'DUMP_TMP_PY 创建失败，程序终止', e)
+            log.error(str(e))
+            return False
+        
+        # 源端导出dmp
+        if src_ora_user != '':
+            sh = '''
+                rm -f ''' + dump_tmp_py + '''/{dump_filename}
+                source ~/.bash_profile
+                export ORACLE_SID={oracle_sid}
+                expdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} schemas={src_ora_user} compression=all cluster=n parallel=4
+            '''
+        else:
+            sh = '''
+                rm -f ''' + dump_tmp_py + '''/{dump_filename}
+                source ~/.bash_profile
+                export ORACLE_SID={oracle_sid}
+                expdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} full=y compression=all cluster=n parallel=4
+            '''
+        sh = sh.replace('{oracle_sid}', src_ora_sid)
+        sh = sh.replace('{dump_filename}', dump_filename)
+        sh = sh.replace('{src_ora_user}', src_ora_user)
+        mess = src_host_conn.exec_script(sh)
+        if 'successfully completed' in mess:
+            log.info('源端', src_host_ip, 'DMP导出成功')
+        else:
+            log.error('源端', src_host_ip, 'DMP导出失败，程序终止')
+            log.error(mess)
+            return False
+        return True
+
     @staticmethod     
-    def expdp_user_scp_impdp(src_host_conn, src_dba_conn, src_ora_sid, src_ora_user, dst_host_conn, dst_dba_conn, dst_ora_sid, dump_tmp_py='/tmp/oracle_dump_py'):
+    def expdp_scp_impdp(src_host_conn, src_dba_conn, src_ora_sid, src_ora_user, dst_host_conn, dst_dba_conn, dst_ora_sid, dump_tmp_py='/tmp/oracle_dump_py'):
         """ 
-        0.需要事项创建好表空间
+        0.需要事项创建好表空间, src_ora_user给入''表示full=y
         1.源端和目标端新建 directory DUMP_TMP_PY /tmp/oracle_dump_py
         2.源端获取 需要导出用户的表空间名称和大小
         3.源端执行 expdp 导出数据文件到 DUMP_TMP_PY
         4.scp文件到目标端 dump_tmp_py
         5.目标端执行 impdp 导入数据文件
         """
+        if src_ora_user == '':
+            src_ora_user = 'all'
         date_str = tools.locatdate()
         dump_filename = src_ora_user + '_' + date_str + '.dmp'
         dump_logname = src_ora_user + '_' + date_str + '.log'
         log = tools.Log('logs/expdp_scp_impdp.log')
-
+        src_ora_user = ''
+        
         # 源端创建DUMP_TMP_PY
         src_host_ip = src_host_conn.host
         try:
             src_dba_conn.run('drop directory DUMP_TMP_PY')
+        except Exception as e:
+            pass
+        try:
             src_dba_conn.run("create directory DUMP_TMP_PY as '" + dump_tmp_py + "'")
             log.info('源端', src_host_ip, 'DUMP_TMP_PY 创建成功')
         except Exception as e:
@@ -373,6 +510,9 @@ class OracleTools:
         dst_host_ip = dst_host_conn.host
         try:
             dst_dba_conn.run('drop directory DUMP_TMP_PY')
+        except Exception as e:
+            pass
+        try:
             dst_dba_conn.run("create directory DUMP_TMP_PY as '" + dump_tmp_py + "'")
             log.info('目标端', dst_host_ip, 'DUMP_TMP_PY 创建成功')
         except Exception as e:
@@ -381,12 +521,20 @@ class OracleTools:
             return False
         
         # 源端导出dmp
-        sh = '''
-            rm -f ''' + dump_tmp_py + '''/{dump_filename}
-            source ~/.bash_profile
-            export ORACLE_SID={oracle_sid}
-            expdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} schemas={src_ora_user} compression=all cluster=n parallel=4
-        '''
+        if src_ora_user != '':
+            sh = '''
+                rm -f ''' + dump_tmp_py + '''/{dump_filename}
+                source ~/.bash_profile
+                export ORACLE_SID={oracle_sid}
+                expdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} schemas={src_ora_user} compression=all cluster=n parallel=4
+            '''
+        else:
+            sh = '''
+                rm -f ''' + dump_tmp_py + '''/{dump_filename}
+                source ~/.bash_profile
+                export ORACLE_SID={oracle_sid}
+                expdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} full=y compression=all cluster=n parallel=4
+            '''
         sh = sh.replace('{oracle_sid}', src_ora_sid)
         sh = sh.replace('{dump_filename}', dump_filename)
         sh = sh.replace('{src_ora_user}', src_ora_user)
@@ -402,7 +550,7 @@ class OracleTools:
         sh = '''
             source ~/.bash_profile
             expect -c "
-            spawn scp -P {dst_host_port} -r ''' + dump_tmp_py + '''/tmp/oracle_dump_py/{dump_filename} {dst_host_user}@{dst_host_ip}:''' + dump_tmp_py + '''/
+            spawn scp -P {dst_host_port} -r ''' + dump_tmp_py + '''/{dump_filename} {dst_host_user}@{dst_host_ip}:''' + dump_tmp_py + '''/
             expect {
                 \\"*assword\\" {set timeout 30; send \\"{dst_host_password}\\r\\";}
                 \\"yes/no\\" {send \\"yes\\r\\"; exp_continue;}
@@ -429,7 +577,7 @@ class OracleTools:
         sh = '''
             source ~/.bash_profile
             export ORACLE_SID={oracle_sid}
-            impdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} logfile={dump_logname} schemas={src_ora_user} transform=segment_attributes:n table_exists_action=replace
+            impdp \\'/ as sysdba \\' directory=DUMP_TMP_PY dumpfile={dump_filename} logfile={dump_logname} full=y transform=segment_attributes:n table_exists_action=replace
         '''
         sh = sh.replace('{oracle_sid}', dst_ora_sid)
         sh = sh.replace('{src_ora_user}', src_ora_user)
@@ -513,7 +661,6 @@ class OracleTools:
     @staticmethod     
     def analysis_ogg_info(host_conn):
         """ 对主机所有找到的ggsci，搜寻全部ogg进程的基本信息 """
-        """ 返回 {'ext_info':[], 'dmp_info':[], 'rep_info':[], 'rep2kafka_info':[]} """
         host = host_conn.host
         tns_list = OracleTools.analysis_tns(host_conn)
         
