@@ -69,8 +69,9 @@ def rm(path):
 
 
 def merge_spaces(s):
-    """ 空格合并，直到不存在多个连续空格 """
+    """ 空格合并(\t会视为空格，\n不会)，直到不存在多个连续空格 """
     s2 = ''
+    s = s.replace('\t', ' ')
     for line in s.split('\n'):
         s2 = s2 + ' '.join(line.split()) + '\n'
     return s2.strip()
@@ -219,5 +220,17 @@ def remove_leading_space(str1):
     return str2
 
 
+def analysis_hosts(host_conn):
+    kv = {}
+    hosts_str = host_conn.exec_command('cat /etc/hosts')
+    for line in hosts_str.split('\n'):
+        line = merge_spaces(line)
+        strs = line.split(' ')
+        if strs[0] != '::1' and strs[0] != '':
+            for idx in range(1, len(strs)):
+                kv[strs[idx]] = strs[0]
+    return kv
+
+
 if __name__ == '__main__':
     main()
